﻿using System;
using System.Collections.Generic;
using System.Text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using iTextSharp.text;
using System.Data;
using System.IO;

namespace PDFUtilitiesNew
{
    class PDFHeaderFooterCreator
    {
        public static string SetPDFHeaderFooter(string filepath, string outputFilepath, string password,
            string headerText,int headerAlignment,string footerText,int footerAlignment)
        {
            try
            {
                PdfReader reader = null;

                if (String.IsNullOrEmpty(password))
                {
                    reader = new PdfReader(filepath);
                }
                else
                {
                    reader = new PdfReader(filepath, Encoding.Default.GetBytes(password));
                }

                string headerPosition = "";

                if (headerAlignment == 0)
                {
                    headerPosition = "Top Center";
                }
                else if (headerAlignment == 1)
                {
                    headerPosition = "Top Left";
                }
                else if (headerAlignment == 2)
                {
                    headerPosition = "Top Right";
                }

                string footerPosition = "";

                if (footerAlignment == 0)
                {
                    footerPosition = "Bottom Center";
                }
                else if (footerAlignment == 1)
                {
                    footerPosition = "Bottom Left";
                }
                else if (footerAlignment == 2)
                {
                    footerPosition = "Bottom Right";
                }

                using (var ms = new MemoryStream(10 * 1024))
                {
                    using (var stamper = new PdfStamper(reader, ms))
                    {
                        for (var i = 1; i <= reader.NumberOfPages; i++)
                        {
                            var dc = stamper.GetOverContent(i);

                            if (headerText != string.Empty)
                            {
                                AddWaterMarkText(
                                    dc,
                                    headerText,
                                    BaseFont.CreateFont(BaseFont.HELVETICA, Encoding.ASCII.EncodingName, false),
                                    14,
                                    0,
                                    new BaseColor(0, 0, 0), headerPosition,
                                    reader.GetPageSizeWithRotation(i));
                            }

                            if (footerText != string.Empty)
                            {
                                AddWaterMarkText(
                                    dc,
                                    footerText,
                                    BaseFont.CreateFont(BaseFont.HELVETICA, Encoding.ASCII.EncodingName, false),
                                    14,
                                    0,
                                    new BaseColor(0, 0, 0), footerPosition,
                                    reader.GetPageSizeWithRotation(i));
                            }
                        }
                        stamper.Close();
                    }

                    System.IO.File.WriteAllBytes(outputFilepath, ms.ToArray());
                }

                reader.Close();
                reader.Dispose();
                reader = null;

                return "";

            }

            catch (Exception ex)
            {
                return "Error could not add Header and Footer of PDF document " + System.IO.Path.GetFileName(filepath) + " " + ex.Message;
            }
        }

        /// <summary>
        /// This method adds watermark text under pdf content
        /// </summary>
        /// <param name="pdfData">pdf content bytes</param>
        /// <param name="watermarkText">text to be shown as watermark</param>
        /// <param name="font">base font</param>
        /// <param name="fontSize">font sieze</param>
        /// <param name="angle">angle at which watermark needs to be shown</param>
        /// <param name="color">water mark color</param>
        /// <param name="realPageSize">pdf page size</param>
        private static void AddWaterMarkText(PdfContentByte pdfData, string watermarkText, BaseFont font, float fontSize, float angle, BaseColor color, string position, Rectangle realPageSize)
        {
            double radians = (angle * Math.PI) / 180;

            var gstate = new PdfGState { FillOpacity = 0.35f, StrokeOpacity = 0.3f };
            pdfData.SaveState();
            pdfData.SetGState(gstate);
            pdfData.SetColorFill(color);
            pdfData.BeginText();
            pdfData.SetFontAndSize(font, fontSize);

            float width = font.GetWidthPoint(watermarkText, fontSize);
            float height = font.GetAscentPoint(watermarkText, fontSize) - font.GetDescentPoint(watermarkText, fontSize);

            if (position == "Middle Center")
            {
                var x = (realPageSize.Left + realPageSize.Right) / 2;
                var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Middle Left")
            {
                //var x = realPageSize.Left + width / 2;

                var x = realPageSize.Left + (float)(Math.Cos(radians) * width) / 2;

                var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Middle Right")
            {
                //var x = realPageSize.Right - width / 2;

                var x = realPageSize.Right - (float)(Math.Cos(radians) * width) / 2;

                var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Top Left")
            {
                //var x = realPageSize.Left + width/2;

                //var y = realPageSize.Top - height;

                var x = realPageSize.Left + (float)(Math.Cos(radians) * width) / 2 + 12;

                var y = realPageSize.Top - (float)(Math.Sin(radians) * width) / 2 - 24;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Top Right")
            {
                //var x = realPageSize.Right - width/2;

                //var y = realPageSize.Top - height;

                var x = realPageSize.Right - (float)(Math.Cos(radians) * width) / 2 - 12;

                var y = realPageSize.Top - (float)(Math.Sin(radians) * width) / 2 - 24;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Top Center")
            {
                var x = (realPageSize.Left + realPageSize.Right) / 2;

                var y = realPageSize.Top - (float)(Math.Sin(radians) * width) / 2 - 24;

                //var y = realPageSize.Top - height;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);

            }
            else if (position == "Bottom Left")
            {

                //var x = realPageSize.Left + width / 2;

                var x = realPageSize.Left + (float)(Math.Cos(radians) * width) / 2 + 12;

                //var y = realPageSize.Bottom + height/2;

                var y = realPageSize.Bottom + (float)(Math.Sin(radians) * width) / 2 + 12;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Bottom Right")
            {
                //var x = realPageSize.Right - width / 2;

                //var y = realPageSize.Bottom + height / 2;

                var x = realPageSize.Right - (float)(Math.Cos(radians) * width) / 2 - 12;

                var y = realPageSize.Bottom + (float)(Math.Sin(radians) * width) / 2 + 12;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Bottom Center")
            {
                var x = (realPageSize.Left + realPageSize.Right) / 2;

                //var y = realPageSize.Bottom + height / 2;                

                var y = realPageSize.Bottom + (float)(Math.Sin(radians) * width) / 2 + 12;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);

            }

            pdfData.EndText();
            pdfData.RestoreState();
        }        
    }
}
